const path = require('path');
const webpack = require('webpack');
const autoprefixer = require('autoprefixer');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const MpPlugin = require('@zhangcheck/mp-webpack-plugin');
const CopyPlugin = require('copy-webpack-plugin');
const cssLoader = require.resolve('css-loader');
const getClientEnvironment = require('./env');
const entries = require('./entry');
const mpuiList = require('@zhangcheck/mp-ui/mp-entry');
const paths = require('./paths');

// const env = getClientEnvironment('');

// 增加页面时，自动编译
const isOptimize = false; // 是否压缩业务代码，开发者工具可能无法完美支持业务代码使用到的 es 特性，建议自己做代码压缩

// todo: 加入mp-ui文档页面
mpuiList(entries);

module.exports = {
  /*watch: true,
  watchOptions: {
    ignored: ['node_modules', 'build', 'templates'],
  },*/
  mode: 'production',
  entry: entries,
  output: {
    path: paths.resolveApp('./build/mp/common'), // 放到小程序代码目录中的 common 目录下
    filename: '[name].js', // 必需字段，不能修改
    library: 'createApp', // 必需字段，不能修改
    libraryExport: 'default', // 必需字段，不能修改
    libraryTarget: 'window', // 必需字段，不能修改
  },
  target: 'web', // 必需字段，不能修改
  optimization: {
    runtimeChunk: false, // 必需字段，不能修改
    splitChunks: {
      // 代码分隔配置，不建议修改
      chunks: 'all',
      minSize: 1000,
      maxSize: 0,
      minChunks: 1,
      maxAsyncRequests: 100,
      maxInitialRequests: 100,
      automaticNameDelimiter: '~',
      name: true,
      cacheGroups: {
        vendors: {
          test: /[\\/]node_modules[\\/]/,
          priority: -10,
        },
        default: {
          minChunks: 2,
          priority: -20,
          reuseExistingChunk: true,
        },
      },
    },

    minimizer: isOptimize
      ? [
          // 压缩CSS
          new OptimizeCSSAssetsPlugin({
            assetNameRegExp: /\.(css|wxss)$/g,
            cssProcessor: require('cssnano'),
            cssProcessorPluginOptions: {
              preset: [
                'default',
                {
                  discardComments: {
                    removeAll: true,
                  },
                  minifySelectors: false, // 因为 wxss 编译器不支持 .some>:first-child 这样格式的代码，所以暂时禁掉这个
                },
              ],
            },
            canPrint: false,
          }),
          // 压缩 js
          new TerserPlugin({
            test: /\.js(\?.*)?$/i,
            parallel: true,
          }),
        ]
      : [],
  },
  module: {
    rules: [
      {
        oneOf: [
          {
            test: /\.less$/,
            use: [
              MiniCssExtractPlugin.loader,
              // 'style-loader',
              'css-loader',
              {
                loader: 'less-loader',
                options: {
                  sourceMap: true,
                  javascriptEnabled: true,
                },
              },
            ],
            // include: paths.antdMobile,
          },
          {
            test: /\.css$/,
            include: [paths.appSrc, paths.mpui],
            exclude: [paths.antdTheme],
            use: [
              MiniCssExtractPlugin.loader,
              {
                loader: require.resolve('css-loader'),
                options: {
                  modules: true,
                  importLoaders: 1,
                },
              },
              {
                loader: require.resolve('postcss-loader'),
                options: {
                  // Necessary for external CSS imports to work
                  // https://github.com/facebookincubator/create-react-app/issues/2677
                  ident: 'postcss',
                  plugins: () => [
                    // Transfer @import rule by inlining content, e.g. @import 'normalize.css'
                    // https://github.com/postcss/postcss-import
                    require('postcss-import')(),
                    // W3C variables, e.g. :root { --color: red; } div { background: var(--color); }
                    // https://github.com/postcss/postcss-custom-properties
                    require('postcss-custom-properties')(),
                    // W3C CSS Custom Media Queries, e.g. @custom-media --small-viewport (max-width: 30em);
                    // https://github.com/postcss/postcss-custom-media
                    require('postcss-custom-media')(),
                    // CSS4 Media Queries, e.g. @media screen and (width >= 500px) and (width <= 1200px) { }
                    // https://github.com/postcss/postcss-media-minmax
                    require('postcss-media-minmax')(),
                    // W3C CSS Custom Selectors, e.g. @custom-selector :--heading h1, h2, h3, h4, h5, h6;
                    // https://github.com/postcss/postcss-custom-selectors
                    // require('postcss-custom-selectors')(),
                    // W3C calc() function, e.g. div { height: calc(100px - 2em); }
                    // https://github.com/postcss/postcss-calc
                    require('postcss-calc')(),
                    // Allows you to nest one style rule inside another
                    // https://github.com/jonathantneal/postcss-nesting
                    require('postcss-nesting')(),
                    // Unwraps nested rules like how Sass does it
                    // https://github.com/postcss/postcss-nested
                    require('postcss-nested')(),
                    // W3C color() function, e.g. div { background: color(red alpha(90%)); }
                    // https://github.com/postcss/postcss-color-function
                    // require('postcss-color-function')(),
                    // Convert CSS shorthand filters to SVG equivalent, e.g. .blur { filter: blur(4px); }
                    // https://github.com/iamvdo/pleeease-filters
                    // require('pleeease-filters')(),
                    // Generate pixel fallback for "rem" units, e.g. div { margin: 2.5rem 2px 3em 100%; }
                    // https://github.com/robwierzbowski/node-pixrem
                    // require('pixrem')(),
                    // W3C CSS Level4 :matches() pseudo class, e.g. p:matches(:first-child, .special) { }
                    // https://github.com/postcss/postcss-selector-matches
                    // require('postcss-selector-matches')(),
                    // Transforms :not() W3C CSS Level 4 pseudo class to :not() CSS Level 3 selectors
                    // https://github.com/postcss/postcss-selector-not
                    // require('postcss-selector-not')(),
                    // Postcss flexbox bug fixer
                    // https://github.com/luisrudge/postcss-flexbugs-fixes
                    require('postcss-flexbugs-fixes')(),
                    // Add vendor prefixes to CSS rules using values from caniuse.com
                    // https://github.com/postcss/autoprefixer
                    // require('autoprefixer')(/* package.json/browserslist */),

                    // require('postcss-animation'),
                    require('postcss-normalize')(),
                    require('@zhangcheck/postcss-rem-to-rpx')(),
                    autoprefixer({
                      browsers: [
                        '>1%',
                        'last 4 versions',
                        'Firefox ESR',
                        'not ie < 9', // React doesn't support IE8 anyway
                      ],
                      flexbox: 'no-2009',
                    }),
                  ],
                },
              },
            ],
          },
          {
            test: /\.css$/,
            use: [
              MiniCssExtractPlugin.loader,
              {
                loader: 'css-loader',
                options: {
                  modules: false,
                  importLoaders: 1,
                },
              },
            ],
          },
          {
            test: /\.(js|jsx|mjs)$/,
            //include: paths.appSrc,
            loader: require.resolve('babel-loader'),
            options: {
              presets: [
                '@babel/preset-env',
                [
                  '@babel/preset-react',
                  // {
                  //   "pragma": "h"
                  // }
                ],
              ],
              plugins: [
                'module:babel-preprocessor',
                '@babel/plugin-proposal-class-properties',
                [
                  '@babel/plugin-proposal-decorators',
                  {
                    legacy: true,
                  },
                ],
                '@babel/plugin-proposal-function-bind',
                '@babel/plugin-proposal-object-rest-spread',
                '@babel/plugin-syntax-dynamic-import',
                // "styled-jsx/babel"
              ],
              // This is a feature of `babel-loader` for webpack (not Babel itself).
              // It enables caching results in ./node_modules/.cache/babel-loader/
              // directory for faster rebuilds.
              cacheDirectory: true,
            },
          },
          {
            test: /\.(png|jpg|gif|svg)$/,
            loader: 'file-loader',
            options: {
              name: '[name].[ext]?[hash]',
            },
          },
        ],
      },
    ],
  },
  resolve: {
    extensions: ['*', '.js', '.vue', '.json'],
  },
  plugins: [
    new webpack.DefinePlugin({
      'process.env.isMiniprogram': process.env.isMiniprogram === 'true', // 注入环境变量，用于业务代码判断
    }),
    // new webpack.DefinePlugin(env.stringified),
    new MiniCssExtractPlugin({
      filename: '[name].wxss',
    }),
    new MpPlugin(require(paths.resolveApp('./config/miniprogram.config'))),
    new CopyPlugin([
      {
        from: paths.resolveApp(`./public/nv-tabbar/*.*`),
        to: '../nv-tabbar/[name].[ext]',
        // ignore: '',
      },
    ]),
  ],
};
