// @flow
import isWx from './isWx'
import history from './history'
import gl from './global'

export function fetch1(url, options) {
  options = options || {}
  let { headers, body, ...rest } = options

  let userInfo
  if (isWx) {
    userInfo = wx.getStorageSync('userInfo')
  } else {
    userInfo = JSON.parse(localStorage.getItem('userInfo'))
  }

  if (!headers) {
    headers = {}
  }
  if (userInfo) {
    headers.Authorization = `Bearer ${userInfo.token}`
  }

  let requestTask

  const ps = new Promise(async (resolve, reject) => {
    const success = res => {
      if (res.status === 401 || res.statusCode === 401) {
        history.blank(`/pages/login/index`)
      } else if (res.status === 200 || res.statusCode === 200) {
        if (options.success) {
          options.success(res)
        }
        resolve([null, res])
      } else {
        if (options.fail) {
          options.fail()
        }
        resolve([res, null])
      }
    }
    const fail = errors => {
      if (options.fail) {
        options.fail(errors)
      }
      resolve([errors, null])
    }
    const complete = res => {
      if (options.complete) {
        options.complete(res)
      }
    }

    if (isWx) {
      requestTask = wx.request({ url, header: headers, data: body, success, fail, complete, ...rest })
    } else {
      if (body) {
        if (options.method === undefined || options.method === 'GET') {
          const parameters = []
          Object.keys(body).forEach(p => {
            if (url.indexOf(`${p}=`) > -1) {
              url = url.replace(new RegExp(`([?&])${p}=[^?&]*`), ($0, $1) => `${$1}${p}=${body[p]}`)
            } else {
              parameters.push(`${p}=${encodeURIComponent(body[p])}`)
            }
          })
          if (parameters.length) {
            url += url.indexOf('?') > -1 ? '&' : '?'
            url += parameters.join('&')
          }
          body = undefined;
        } else if (options.method === 'POST' || options.method === 'PUT' || options.method === 'DELETE') {
          if (typeof body === 'object') {
            if (headers['Content-Type'] === 'application/x-www-form-urlencoded') {
              // 如果不是formdata
              if (!body.append) {
                const ps = []
                Object.keys(body).forEach(p => {
                  if (typeof body[p] === 'string') {
                    ps.push(`${p}=${encodeURIComponent(body[p])}`)
                  } else {
                    ps.push(`${p}=${encodeURIComponent(JSON.stringify(body[p]))}`)
                  }
                })
                if (ps.length > 0) {
                  body = ps.join('&')
                }
              }
            } else if ((headers['Content-Type'] === undefined || headers['Content-Type'] === 'application/json') && !body.append) {
              headers['Content-Type'] = 'application/json'
              if (Object.keys(body).length > 0) {
                body = JSON.stringify(body)
              }
            }
          }
        }
      }
      try {
        const res = await fetch(url, {
          body,
          method: options.method || 'GET',
          headers,
        })
        res.data = await res.json()
        success(res)
      } catch (e) {
        fail(e)
      }
    }
  })

  return ps
}
export default fetch1
export const SERVICE = gl.SERVICE;

