import React, { useEffect, useRef } from 'react';

/**
 * 计时器
 * @param callback
 * @param delay
 * @param seed 通过seed可以重置计时器
 */
function useInterval(callback, delay, seed) {
  const savedCallback = useRef();

  // Remember the latest callback.
  useEffect(() => {
    savedCallback.current = callback;
  }, [callback]);

  // Set up the interval.
  useEffect(() => {
    function tick() {
      savedCallback.current();
    }
    if (delay != null && seed) {
      let id = setInterval(tick, delay);
      return () => clearInterval(id);
    }
  }, [delay, seed]);
}

export default useInterval;
